################################################################################
## R Code for replication of empirical illustration
## Tested on R x64 3.1.2.
## Oct 31, 2016
################################################################################


# Read data set from web repository
rhc <- read.csv("http://biostat.mc.vanderbilt.edu/wiki/pub/Main/DataSets/rhc.csv")[,-c(1,63)]
rhc[,"death"] <- as.numeric(rhc[,"death"]) -1
attach(rhc)

# Estimate propensity score
glmFull <- glm(formula = swang1 ~ age + sex + race + edu + income + ninsclas + cat1 + das2d3pc + dnr1 + ca + surv2md1 + aps1 + scoma1 + wtkilo1 + temp1 + meanbp1 + resp1 + hrt1 + pafi1 + paco21 + ph1 + wblc1 + hema1 + sod1 + pot1 + crea1 + bili1 + alb1 + resp + card + neuro + gastr + renal + meta + hema + seps + trauma + ortho + cardiohx + chfhx + dementhx + psychhx + chrpulhx + renalhx + liverhx + gibledhx + malighx + immunhx + transhx + amihx,
               family  = binomial(link = "logit"),
               data    = rhc)
pscore <- predict(glmFull,type="response")

# Plot Figure 2
par(mfrow=c(1,2))
par(mai=c(0.45, .45, 0.1, 0.1))
par(las=1,cex=.8)
hist(pscore[swang1=="RHC"], main="")
hist(pscore[swang1!="RHC"], main="")

# Discretize propensity score and calculate the various elements of the CI formula
ps.disc20 <- as.factor(floor(pscore*20)/20)
a1 <- lm(death~ps.disc20, subset=(swang1=="RHC"))
a0 <- lm(death~ps.disc20, subset=(swang1!="RHC"))
n <- length(death)

w1 <- predict(a1,rhc)
f1 <- table(w1)/n
n1 <- aggregate(death[swang1=="RHC"],list(w1[swang1=="RHC"]),length)[,2]
mu1 <- aggregate(death[swang1=="RHC"],list(w1[swang1=="RHC"]),mean)[,2]
sig1 <- aggregate(death[swang1=="RHC"],list(w1[swang1=="RHC"]),var)[,2]


w0 <- predict(a0,rhc)
f0 <- table(w0)/n
n0 <- aggregate(death[swang1!="RHC"],list(w0[swang1!="RHC"]),length)[,2]
mu0 <- aggregate(death[swang1!="RHC"],list(w0[swang1!="RHC"]),mean)[,2]
sig0 <- aggregate(death[swang1!="RHC"],list(w0[swang1!="RHC"]),var)[,2]

ate.point <- sum(f1*mu1) - sum(f0*mu0)
omegaA <- (sum(f1^2*sig1/n1) + sum(f0^2*sig0/n0))
degfree1 <- min(c(n1,n0))-1
alpha <- .05
rho <- sqrt((sum((qt(1-alpha/2, df = n1-1)/qt(1-alpha/2, df = degfree1))^2 *f1^2*sig1/n1) + sum((qt(1-alpha/2, df = n0-1)/qt(1-alpha/2, df = degfree1))^2 * f0^2*sig0/n0))/omegaA)

# ATE point estimate and standard error (Table 1, lines 1-2)
c(ate.point , sqrt(omegaA))

# Conventional and robust 95% critical value (Table 1, lines 3-4)
c(qnorm(.975),qt(.975,degfree1)*rho )

# Conventional and robust 95% confidence interval (Table 1, lines 4-5)
ate.point + c(-1,1)* qnorm(.975)*sqrt(omegaA)
ate.point + c(-1,1)* qt(.975,degfree1)*rho*sqrt(omegaA)
