################################################################################
## R Code for replication of simulation study
## Tested on R x64 3.1.2.
## Oct 31, 2016
################################################################################


################################################################################
## Part I
## Define function for calculating various types of CIs
################################################################################

library(sandwich)
ate <- function(y,x,d,alpha=.1, h0 = 0, type = "ztest", true.sig = NA) {

 n <- length(y)
 fx <- table(x)/n
 ndx <- aggregate(y,list(x,d),length)[,3]
 mu <- aggregate(y,list(x,d),mean)[,3]
 sig <- aggregate(y,list(x,d),var)[,3]
 pd <- aggregate(d,list(x),mean)[,2]
 pd <- c(1-pd,pd)

 output <- rep(NA,8)

 if (length(ndx)==2*length(fx) & min(ndx) > 1) {

  if (type=="boot") {
    ate <- sum(mu*c(-fx,fx))
    omegaA <- sum( (c(fx,fx)*sig/pd))/n
    tstat <- (ate - h0)/sqrt(omegaA)

    wtd.bs <- function(ww) {
       ate.star <- sum((aggregate(y*ww,list(x,d),sum)[,3]/aggregate(ww,list(x,d),sum)[,3])*c(-fx,fx))
       sig.star <- ((aggregate(y^2*ww,list(x,d),sum)[,3]/aggregate(ww,list(x,d),sum)[,3])-(aggregate(y*ww,list(x,d),sum)[,3]/aggregate(ww,list(x,d),sum)[,3])^2)*(ndx)/(ndx-1)
       omegaA.star <- sum( (c(fx,fx)*sig.star/pd))/n
       (ate.star-ate)/sqrt(omegaA.star)
       }
    tstat.star <- apply(matrix(rexp(n*19),ncol=n), 1 ,wtd.bs)     ###
    pval <- mean(abs(tstat.star)> abs(tstat))
    critval <- quantile(abs(tstat.star),1-alpha/2)
    ci.lower <- ate - critval*sqrt(omegaA )
    ci.upper <- ate + critval*sqrt(omegaA )
    output <- c(ate,sqrt(omegaA), tstat, critval, pval, ci.lower, ci.upper, NA)

  }

  if (type=="reg") {
   mod1 <- lm(y~x, subset=(d==1))
   mod0 <- lm(y~x, subset=(d==0))
   ate <- coef(mod1)[1]-coef(mod0)[1] + mean(x)*(coef(mod1)[2]-coef(mod0)[2])
   vec <- c(1,mean(x))
   omegaA <- as.vector(t(vec)%*%vcovHC(mod1, type = "HC1")%*%vec + t(vec)%*%vcovHC(mod0, type = "HC1")%*%vec)
   tstat <- (ate - h0)/sqrt(omegaA)
   degfree <- Inf
   critval <- qnorm(1-alpha/2)
   ci.lower <- ate - critval*sqrt(omegaA )
   ci.upper <- ate + critval*sqrt(omegaA )
   pval <- 2*(1-pnorm(abs(tstat)))
   output <- c(ate,sqrt(omegaA), tstat, critval, pval, ci.lower, ci.upper, degfree)
  }

  if (type=="ipw") {

   mod <- glm(d~x, family=binomial(link="probit"))
   pscore <- predict(mod,type="response")
   k.hat <- (d-pscore)*y/(pscore*(1-pscore))
   ate <- mean(k.hat)
   pd <- aggregate(pscore,list(x),mean)[,2]
   pd <- c(1-pd,pd)
   omegaA <- sum( (c(fx,fx)*sig/pd))/n
   tstat <- (ate - h0)/sqrt(omegaA)
   degfree <- Inf
   critval <- qnorm(1-alpha/2)
   ci.lower <- ate - critval*sqrt(omegaA )
   ci.upper <- ate + critval*sqrt(omegaA )
   pval <- 2*(1-pnorm(abs(tstat)))
   output <- c(ate,sqrt(omegaA), tstat, critval, pval, ci.lower, ci.upper, degfree)
  }

  if (type=="cons") {
   ate <- sum(mu*c(-fx,fx))
   omegaA <- sum( (c(fx,fx)*sig/pd))/n
   tstat <- (ate - h0)/sqrt(omegaA)
   degfree <- min(ndx-1)
   critval <- qt(1-alpha/2, df = degfree)
   rho <- sum( (qt(1-alpha/2, df = ndx-1)/critval)^2 * (c(fx,fx)*sig/pd)) / sum( (c(fx,fx)*sig/pd))
   ci.lower <- ate - critval*sqrt(rho* omegaA )
   ci.upper <- ate + critval*sqrt(rho* omegaA )
   pval <- 2*(1-pt(abs(tstat)/sqrt(rho),df=degfree))
   output <- c(ate,sqrt(omegaA), tstat, critval, pval, ci.lower, ci.upper, degfree)
  }

  if (type=="ztest") {
   ate <- sum(mu*c(-fx,fx))
   omegaA <- sum( (c(fx,fx)*true.sig/pd))/n
   tstat <- (ate - h0)/sqrt(omegaA)
   degfree <- Inf
   critval <- qt(1-alpha/2, df = degfree)
   ci.lower <- ate - critval*sqrt(omegaA)
   ci.upper <- ate + critval*sqrt(omegaA)
   pval <- 2*(1-pt(abs(tstat),df=degfree))
   output <- c(ate,sqrt(omegaA), tstat, critval, pval, ci.lower, ci.upper, degfree)
  }
 }
 names(output) <- c("ATE", "S.E.", "t-Stat (H0)","CV (H0)","p-Value","CI.lower","CI.upper","Eff.DF")
 output
}


################################################################################
## Part II
## Define function that generates data and calls CI function
################################################################################

sims <- function(aaa, mmm) {

n <- 10*mmm

out <- rep(0,5)
ll <- rep(0,5)

 xlevels <- 10
 x.dens <- rep(1/xlevels,xlevels)

 mu1 <- (1:xlevels)^1.2
 mu0 <- (xlevels:1)^0
 ate.true <- sum(x.dens*(mu1-mu0))

 sig1 <- 1+3*(1/3)^((xlevels:1)-1)
 sig0 <- (1:xlevels)^0

 x <- rep(1:xlevels,rep(n/xlevels,xlevels))
 d <- rep(rep(0:1,xlevels),c(rep(.5*n/xlevels,2*(xlevels-1)),mmm-aaa,aaa))


 y1 <- mu1[x] + sqrt(sig1[x]) * rnorm(n)
 y0 <- mu0[x] + sqrt(sig0[x]) * rnorm(n)

 y <- d*y1 + (1-d)*y0

 est <- ate(y,x,d,alpha=.05, h0 = ate.true,type="cons")
 out[1] <- (1-pnorm(abs(est[3])))*2
 tstat <- est[3]
  ll[1] <- 2*est[2]*qnorm(.975)
 out[2] <- est[5]
  ll[2] <- abs(est[7]-est[6])
 est <- ate(y,x,d,alpha=.05, h0 = ate.true,type="boot",true.sig=c(sig0,sig1))
 out[3] <- est[5]
  ll[3] <- abs(est[7]-est[6])
 est <- ate(y,x,d,alpha=.05, h0 = ate.true,type="reg",true.sig=c(sig0,sig1))
 out[4] <- est[5]
  ll[4] <- abs(est[7]-est[6])
 est <- ate(y,x,d,alpha=.05, h0 = ate.true,type="ztest",true.sig=c(sig0,sig1))
 out[5] <- est[5]
  ll[5] <- abs(est[7]-est[6])

 c(out,ll,tstat)
}

################################################################################
## Part III
## Run simulations (and produce preliminary version of Figure 1)
################################################################################

a <- c(100,50,30,15,10,8,5,4,3,2)

reps <- 10000
ra <- matrix(0,ncol=length(a),nrow=reps)
za <- matrix(0,ncol=length(a),nrow=reps)
ba <- matrix(0,ncol=length(a),nrow=reps)
ca <- matrix(0,ncol=length(a),nrow=reps)
ga <- matrix(0,ncol=length(a),nrow=reps)

tstat <- matrix(0,ncol=length(a),nrow=reps)

ral <- matrix(0,ncol=length(a),nrow=reps)
zal <- matrix(0,ncol=length(a),nrow=reps)
bal <- matrix(0,ncol=length(a),nrow=reps)
cal <- matrix(0,ncol=length(a),nrow=reps)
gal <- matrix(0,ncol=length(a),nrow=reps)

set.seed(12345)

for (j in 1:reps){
for (i in 1:length(a)) {
 out1 <- sims(a[i],200)
 za[j,i] <- out1[5]
 ra[j,i] <- out1[4]
 ba[j,i] <- out1[3]
 ca[j,i] <- out1[2]
 ga[j,i] <- out1[1]
 
 tstat[j,i] <- out1[11]
 
 zal[j,i] <- quantile(abs(tstat[1:j,i]),.95)*out1[6]/qnorm(.975)
 ral[j,i] <- out1[9]
 bal[j,i] <- out1[8]
 cal[j,i] <- out1[7]
 gal[j,i] <- out1[6]
}

## The simulations take a while, so we plot a preliminary version of Figure 1
## every 50 runs to monitor the progress.

if (floor(j/50)==j/50){

ra1 <- apply(ra[1:j,], 2, function(a) {mean(a<.05)} )
ba1 <- apply(ba[1:j,], 2, function(a) {mean(a<.05)} )
ca1 <- apply(ca[1:j,], 2, function(a) {mean(a<.05)} )
ga1 <- apply(ga[1:j,], 2, function(a) {mean(a<.05)} )
za1 <- apply(ga[1:j,], 2, function(a) {mean(a<quantile(a,.05))} )

par(mfrow=c(1,2))
par(mai=c(0.45, .45, 0.1, 0.1))
par(las=1,cex=.8)
plot(a/(2*max(a)),1-ga1,type="l", lwd=2,ylim = c(.75,1),log="x")
lines(a/(2*max(a)),1-ca1,lty=2,lwd=2)
lines(a/(2*max(a)),1-ba1,lty=3,lwd=2)
lines(a/(2*max(a)),1-ra1,lty=4,lwd=2)
lines(a/(2*max(a)),1-za1,lty=5,lwd=2)
abline(h=.95,lty=2)

za1l <- apply(zal[1:j,], 2, median )
ra1l <- apply(ral[1:j,], 2, median )
ba1l <- apply(bal[1:j,], 2, median )
ca1l <- apply(cal[1:j,], 2, median )
ga1l <- apply(gal[1:j,], 2, median )

plot(a/(2*max(a)),ga1l,type="l", lwd=2,ylim = c(.15,3),log="xy")
lines(a/(2*max(a)),ca1l,lty=2,lwd=2)
lines(a/(2*max(a)),ba1l,lty=3,lwd=2)
lines(a/(2*max(a)),ra1l,lty=4,lwd=2)
lines(a/(2*max(a)),za1l,lty=5,lwd=2)

print(paste(round(j/reps,4)*100,"% of simulation runs completed"))
}
}

################################################################################
## Part IV
## Produce final version of Figure 1
################################################################################


par(mfrow=c(1,2))
par(mai=c(0.45, .45, 0.1, 0.1))
par(las=1,cex=.8)
plot(a/(2*max(a)),1-ga1,type="l", lwd=2,ylim = c(.75,1),log="x")
lines(a/(2*max(a)),1-ca1,lty=2,lwd=2)
lines(a/(2*max(a)),1-ba1,lty=3,lwd=2)
lines(a/(2*max(a)),1-ra1,lty=4,lwd=2)
lines(a/(2*max(a)),1-za1,lty=5,lwd=2)

za1l <- apply(zal[1:j,], 2, mean )
ra1l <- apply(ral[1:j,], 2, mean )
ba1l <- apply(bal[1:j,], 2, mean )
ca1l <- apply(cal[1:j,], 2, mean )
ga1l <- apply(gal[1:j,], 2, mean )

plot(a/(2*max(a)),ga1l,type="l", lwd=2,ylim = c(.15,3),log="xy")
lines(a/(2*max(a)),ca1l,lty=2,lwd=2)
lines(a/(2*max(a)),ba1l,lty=3,lwd=2)
lines(a/(2*max(a)),ra1l,lty=4,lwd=2)
lines(a/(2*max(a)),za1l,lty=5,lwd=2)
